/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.Configuration.rules.checkers;

import com.fasterxml.jackson.core.JsonFactory;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonToken;
import com.fasterxml.jackson.databind.MappingJsonFactory;
import org.sonar.api.batch.fs.InputFile;
import org.sonar.api.batch.sensor.SensorContext;
import org.sonar.api.batch.sensor.issue.NewIssue;
import org.sonar.api.rule.RuleKey;
import org.sonar.check.Rule;

import java.io.*;
import java.util.Scanner;


/**
 * 通过用户名口令、数据证书等其他手段对用户身份进行验证。
 *
 * @author WuHaoYang
 * @date 2024/1/22
 */
@Rule(key = "ConfigurationFileChecker")
public class ConfigurationFileChecker implements ConfigCheck{


    public void execute(SensorContext context, InputFile inputFile, RuleKey ruleKey){
        //文件名称
        String filename = inputFile.filename();
        System.out.println("[ConfigurationFileChecker]>>>>>" + filename);

        //校验文件后缀
        if (filename.endsWith(".properties")) {
            try {
                File file = new File(inputFile.absolutePath());
                try (Scanner scanner = new Scanner(file)) {
                    int lineNum = 1;
                    while (scanner.hasNextLine()) {
                        String line = scanner.nextLine();

                        String[] keyValue = line.split("="); // 分割行的内容

                        // 检查键是否为 "password" 且其对应的值不为空
                        if (keyValue.length == 2 && keyValue[0].trim().equals("password")
                                && !keyValue[1].trim().isEmpty()) {
                            NewIssue newIssue = context.newIssue();
                            newIssue.forRule(ruleKey)
                                    .at(newIssue.newLocation()
                                            .on(inputFile)
                                            .at(inputFile.selectLine(lineNum)))
                                    .save();
                        }
                        lineNum++;
                    }
                }
            } catch (FileNotFoundException e) {
                System.out.println("文件未找到: " + e.getMessage());
            }
        }

        if (filename.endsWith(".ini")) {
            // 获取当前输入文件的绝对路径
            File file1 = inputFile.file();

            try (BufferedReader br = new BufferedReader(new FileReader(file1))) {
                String line;
                int lineNum = 1;
                while ((line = br.readLine()) != null) {
                    if (line.trim().startsWith("password")) {
                        NewIssue newIssue = context.newIssue();
                        newIssue.forRule(ruleKey)
                                .at(newIssue.newLocation()
                                        .on(inputFile)
                                        .at(inputFile.selectLine(lineNum)))
                                .save();
                        break;
                    }
                    lineNum++;
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        if (filename.endsWith(".conf")) {
            // 获取当前输入文件的绝对路径
            File file1 = inputFile.file();

            int lineNum = 0;  // 初始化行数

            try {
                BufferedReader br = new BufferedReader(new FileReader(file1));
                String line;

                while ((line = br.readLine()) != null) {
                    lineNum++;  // 递增行数

                    String[] keyValue = line.split("=");  // 分割行的内容

                    // 进行基本有效性检查 && keyValue[0]可能包含前导空格, keyValue[1]可能包含尾随空格
                    if (keyValue.length == 2 && keyValue[0].trim().equals("password")) {

                        // 如果键是 "password", 输出其值
                        NewIssue newIssue = context.newIssue();
                        newIssue
                                .forRule(ruleKey)
                                .at(newIssue.newLocation()
                                        .on(inputFile)
                                        .at(inputFile.selectLine(lineNum)))
                                .save();
                    }
                }
                br.close();
            } catch(FileNotFoundException ex) {
                ex.printStackTrace();
            } catch(IOException ex) {
                ex.printStackTrace();
            }
        }

        if (filename.endsWith(".xml") && !filename.equals("pom.xml")) {
            // 获取当前输入文件的绝对路径
            File file1 = inputFile.file();
            File absoluteFile = file1.getAbsoluteFile();

            BufferedReader reader = null;
            int lineNumber = 0;
            try {
                reader = new BufferedReader(new InputStreamReader(new FileInputStream(file1), "UTF-8"));
                String line;
                while ((line = reader.readLine()) != null) {
                    lineNumber++;
                    if (line.contains("<password>")) {
                        // 创建和保存issue
                        NewIssue newIssue = context.newIssue();
                        newIssue
                                .forRule(ruleKey)
                                .at(newIssue.newLocation().on(inputFile)
                                        .at(inputFile.selectLine(lineNumber)))
                                .save();
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (reader != null) {
                    try { reader.close(); } catch (IOException e) { e.printStackTrace(); }
                }
            }
        }

        if (filename.endsWith(".json")) {
            try {
                // 获取当前输入文件的绝对路径
                File file1 = inputFile.file();

                JsonFactory f = new MappingJsonFactory();
                JsonParser jp = f.createParser(file1);
                JsonToken current;

                current = jp.nextToken();
                if (current != JsonToken.START_OBJECT) {
                    return;
                }

                while (jp.nextToken() != JsonToken.END_OBJECT) {
                    String fieldName = jp.getCurrentName();
                    current = jp.nextToken();

                    if ("password".equals(fieldName)) {

                        int lineNum = jp.getCurrentLocation().getLineNr();

                        String passwordValue = jp.getText();

                        NewIssue newIssue = context.newIssue();
                        newIssue
                                .forRule(ruleKey)
                                .at(newIssue.newLocation()
                                        .on(inputFile)
                                        .at(inputFile.selectLine(lineNum)))
                                .save();
                        break;
                    }
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        if (filename.endsWith(".yml")) {
            File file = inputFile.file();
            try (BufferedReader reader = new BufferedReader(new FileReader(file))) {
                String line;
                int lineNumber = 0;

                while ((line = reader.readLine()) != null) {
                    lineNumber++;

                    if (line.trim().startsWith("password:")) {
                        // 创建并保存issue
                        NewIssue newIssue = context.newIssue();
                        newIssue
                                .forRule(ruleKey)
                                .at(newIssue.newLocation().on(inputFile)
                                        .at(inputFile.selectLine(lineNumber)))
                                .save();
                    }
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

}
