/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：28所 C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx.rules.checkers;


import com.sonar.cxx.sslr.api.AstNode;
import com.sonar.cxx.sslr.api.GenericTokenType;
import com.sonar.cxx.sslr.api.Grammar;
import org.sonar.check.Priority;
import org.sonar.check.Rule;
import org.sonar.cxx.parser.CxxGrammarImpl;
import org.sonar.cxx.squidbridge.annotations.ActivatedByDefault;
import org.sonar.cxx.squidbridge.annotations.SqaleConstantRemediation;
import org.sonar.cxx.squidbridge.checks.SquidCheck;

import java.util.HashSet;
import java.util.Set;


/**
 * 在访问文件或目录前对路径名进行验证
 *
 * @author Wuhaoyang
 * @date 2024/1/6
 */

@Rule(key = "FileAccessChecker", name = "在访问文件或目录前对路径名进行验证", description = "在访问文件或目录前对路径名进行验证", priority = Priority.INFO, tags = {"28suo"})
@ActivatedByDefault
@SqaleConstantRemediation("5min")
public class FileAccessChecker extends SquidCheck<Grammar> {

    private Set<String> conditionVariables;

    @Override
    public void init() {
        subscribeTo(CxxGrammarImpl.selectionStatement);
        subscribeTo(CxxGrammarImpl.declaration);
        conditionVariables = new HashSet<>();
    }

    @Override
    public void visitNode(AstNode node) {
        if (node.is(CxxGrammarImpl.selectionStatement)) {
            handleIfStatement(node);
        } else if (node.is(CxxGrammarImpl.declaration)) {
            addAllIdentifiers(node);
            handleDeclaration(node);
        }
    }


    private void handleDeclaration(AstNode node) {
        AstNode ifstreamNode = findIfstreamNode(node);
        if (ifstreamNode != null) {
            handleIfstreamDeclaration(ifstreamNode);
        }
    }

    private AstNode findIfstreamNode(AstNode node) {
        if (node.getToken() != null && "ifstream".equals(node.getTokenOriginalValue())) {
            return node;
        }
        for (AstNode child : node.getChildren()) {
            AstNode result = findIfstreamNode(child);
            if (result != null) {
                return result;
            }
        }
        return null;
    }


    private void handleIfStatement(AstNode node) {
        AstNode conditionNode = node.getFirstDescendant(CxxGrammarImpl.condition);
        if (conditionNode != null) {
            addAllIdentifiers(conditionNode);
        }
    }

    private void addAllIdentifiers(AstNode node) {
        for (AstNode child : node.getChildren()) {
            if (child.getType().equals(CxxGrammarImpl.selectionStatement)) {
                // 找到 if 语句节点
                AstNode conditionNode = child.getFirstDescendant(CxxGrammarImpl.condition);
                // 找到条件部分的节点
                AstNode identifierNode = conditionNode.getFirstDescendant(GenericTokenType.IDENTIFIER);
                // 找到代表标识符的节点
                if (identifierNode != null) {
                    String codeInsideIf = identifierNode.getTokenValue();
                    // 获取标识符节点的值
                    conditionVariables.add(codeInsideIf);
                }
            } else {
                addAllIdentifiers(child);
            }
        }
    }

    private void handleIfstreamDeclaration(AstNode ifstreamNode) {
        AstNode parameterDeclarationNode = ifstreamNode.getFirstDescendant(CxxGrammarImpl.parameterDeclaration);
        if (parameterDeclarationNode != null) {
            AstNode identifierNode = parameterDeclarationNode.getFirstDescendant(GenericTokenType.IDENTIFIER);
            if (identifierNode != null) {
                String ifstreamParam = identifierNode.getTokenOriginalValue();
                if (!conditionVariables.contains(ifstreamParam)) {
                    getContext().createLineViolation(this, "在访问文件或目录前需要对路径名进行验证", ifstreamNode);
                }
            }
        }
    }

}
