/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：28所 C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx.rules.checkers;


import com.sonar.cxx.sslr.api.AstNode;
import com.sonar.cxx.sslr.api.Grammar;
import org.sonar.check.Priority;
import org.sonar.check.Rule;
import org.sonar.cxx.parser.CxxGrammarImpl;
import org.sonar.cxx.squidbridge.annotations.ActivatedByDefault;
import org.sonar.cxx.squidbridge.annotations.SqaleConstantRemediation;
import org.sonar.cxx.squidbridge.checks.SquidCheck;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 使用VirtualLock()函数锁定存放敏感信息的内存
 *
 * @author WuHaoYang
 * @date 2024/1/6
 */

@Rule(key = "VirtualLockUsageChecker", name = "使用VirtualLock()函数锁定存放敏感信息的内存", description = "使用VirtualLock()函数锁定存放敏感信息的内存", priority = Priority.INFO, tags = {"28suo"})
@ActivatedByDefault
@SqaleConstantRemediation("5min")
public class VirtualLockUsageChecker extends SquidCheck<Grammar> {

    private List<String> keywords = Arrays.asList("add", "keyword2", "keyword3");
    private Map<String, Map<String, AstNode>> caches = new HashMap<>();

    @Override
    public void init() {
        subscribeTo(CxxGrammarImpl.declarationStatement);
    }

    @Override
    public void visitNode(AstNode astNode) {
        String varName = astNode.getFirstDescendant(CxxGrammarImpl.declaratorId).getTokenOriginalValue();

        for (String keyword : keywords) {
            if (varName.equals(keyword) && (!caches.equals(keyword) || !caches.get(keyword).equals(varName))) {
                caches.putIfAbsent(keyword, new HashMap<>());
                caches.get(keyword).put(varName, astNode);
                processNode(astNode, keyword);
            }
        }
    }

    private void processNode(AstNode astNode, String keyword) {
        String varName = astNode.getFirstDescendant(CxxGrammarImpl.declaratorId).getTokenOriginalValue();

        AstNode currentNode = astNode.getNextAstNode();
        while (currentNode != null) {
            AstNode callNode = currentNode.getFirstDescendant(CxxGrammarImpl.postfixExpression);
            if (callNode != null && callNode.getTokenOriginalValue().equalsIgnoreCase("VirtualLock")) {
                List<AstNode> paramList = callNode.getDescendants(CxxGrammarImpl.expressionList);
                if (paramList.stream().anyMatch(item -> item.getTokenOriginalValue().contains(keyword))) {
                    caches.get(keyword).remove(varName);
                    break;
                }
            }
            currentNode = currentNode.getNextSibling();
        }
    }

    @Override
    public void leaveFile(AstNode astNode) {
        reportViolations();
    }

    private void reportViolations() {
        caches.values().forEach(cache ->
                cache.values().forEach(item -> {
                    getContext().createLineViolation(this, "特定字段未使用VirtualLock()函数锁定存放敏感信息的内存", item);
                })
        );
        caches.clear();
    }
}
