/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：28所 C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.sonar.api.batch.fs.InputFile;
import org.sonar.api.batch.fs.internal.DefaultInputFile;
import org.sonar.api.batch.rule.ActiveRules;
import org.sonar.api.batch.rule.CheckFactory;
import org.sonar.api.batch.sensor.highlighting.TypeOfText;
import org.sonar.api.batch.sensor.internal.SensorContextTester;
import org.sonar.api.batch.sensor.issue.internal.DefaultNoSonarFilter;
import org.sonar.api.measures.FileLinesContext;
import org.sonar.api.measures.FileLinesContextFactory;

import java.io.File;
import java.io.IOException;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

class CxxHighlighterTest {

  private CxxSquidSensor sensor;
  private SensorContextTester context;
  private DefaultInputFile inputFile;

  @BeforeEach
  public void scanFile() throws IOException {
    ActiveRules rules = mock(ActiveRules.class);
    var checkFactory = new CheckFactory(rules);
    FileLinesContextFactory fileLinesContextFactory = mock(FileLinesContextFactory.class);
    FileLinesContext fileLinesContext = mock(FileLinesContext.class);
    when(fileLinesContextFactory.createFor(Mockito.any(InputFile.class))).thenReturn(fileLinesContext);

    File baseDir = TestUtils.loadResource("/com/keyware/sonar/cxx");
    inputFile = TestUtils.buildInputFile(baseDir, "highlighter.cc");
    context = SensorContextTester.create(baseDir);
    context.fileSystem().add(inputFile);

    sensor = new CxxSquidSensor(fileLinesContextFactory, checkFactory, new DefaultNoSonarFilter(), null);
    sensor.execute(context);
  }

  @Test
  @SuppressWarnings("squid:S2699") // ... checkOnRange contains the assertion
  void keyword() {

    checkOnRange(55, 0, 4, TypeOfText.KEYWORD);  // void
    checkOnRange(57, 3, 4, TypeOfText.KEYWORD);  // auto
    checkOnRange(59, 3, 4, TypeOfText.KEYWORD);  // auto
    checkOnRange(62, 3, 3, TypeOfText.KEYWORD);  // for
    checkOnRange(62, 7, 5, TypeOfText.KEYWORD);  // const
    checkOnRange(62, 13, 4, TypeOfText.KEYWORD); // auto
    checkOnRange(64, 6, 2, TypeOfText.KEYWORD);  // if
  }

  @Test
  @SuppressWarnings("squid:S2699") // ... checkOnRange contains the assertion
  void stringLiteral() {

    checkOnRange(49, 19, 7, TypeOfText.STRING);  // "hello"
    checkOnRange(50, 19, 18, TypeOfText.STRING); // "hello\tworld\r\n"
    checkOnRange(73, 32, 24, TypeOfText.STRING); // R"([.^$|()\[\]{}*+?\\])"

    checkOnRange(83, 24, 5, TypeOfText.STRING); // "..."
    checkOnRange(84, 24, 7, TypeOfText.STRING); // u8"..."
    checkOnRange(85, 24, 6, TypeOfText.STRING); // L"..."
    checkOnRange(86, 24, 6, TypeOfText.STRING); // u"..."
    checkOnRange(87, 24, 6, TypeOfText.STRING); // U"..."

    // "hello" " world"
    checkOnRange(89, 24, 7, TypeOfText.STRING);
    checkOnRange(89, 32, 8, TypeOfText.STRING);

    // u"" "hello world"
    checkOnRange(90, 24, 3, TypeOfText.STRING);
    checkOnRange(90, 28, 13, TypeOfText.STRING);

    // /*comment1*/ u"" /*comment2*/ "hello world" /*comment3*/; // issue #996
    checkOnRange(91, 24, 12, TypeOfText.COMMENT);
    checkOnRange(91, 37, 3, TypeOfText.STRING);
    checkOnRange(91, 41, 12, TypeOfText.COMMENT);
    checkOnRange(91, 54, 13, TypeOfText.STRING);
    checkOnRange(91, 68, 12, TypeOfText.COMMENT);
    checkOnRange(91, 82, 13, TypeOfText.COMMENT);

    // /*comment4*/ "hello"
    // /*comment5*/ " world" /*comment6*/;
    checkOnRange(93, 24, 12, TypeOfText.COMMENT);
    checkOnRange(93, 37, 7, TypeOfText.STRING);
    checkOnRange(94, 24, 12, TypeOfText.COMMENT);
    checkOnRange(94, 37, 8, TypeOfText.STRING);
    checkOnRange(94, 46, 12, TypeOfText.COMMENT);

    // "hello"
    // "Mary"
    // "Lou";
    checkOnRange(96, 25, 7, TypeOfText.STRING);
    checkOnRange(97, 25, 6, TypeOfText.STRING);
    checkOnRange(98, 25, 5, TypeOfText.STRING);

    // R"(\r\n      Hello World!\r\n   )" issue #1768
    check(103, 14, TypeOfText.STRING);
  }

  @Test
  @SuppressWarnings("squid:S2699") // ... checkOnRange contains the assertion
  void character() {

    checkOnRange(46, 10, 3, TypeOfText.STRING); // 'x'
    checkOnRange(47, 10, 4, TypeOfText.STRING); // '\t'
  }

  @Test
  @SuppressWarnings("squid:S2699") // ... checkOnRange contains the assertion
  void comment() {

    check(1, 0, TypeOfText.COMMENT);
    /*\r\n comment\r\n*/
    check(3, 1, TypeOfText.COMMENT);

    checkOnRange(5, 0, 2, TypeOfText.COMMENT);    //
    checkOnRange(6, 0, 10, TypeOfText.COMMENT);   // comment
    checkOnRange(7, 0, 2, TypeOfText.COMMENT);    //

    checkOnRange(57, 22, 10, TypeOfText.COMMENT); // comment
    checkOnRange(58, 3, 10, TypeOfText.COMMENT);  // comment
    checkOnRange(61, 3, 13, TypeOfText.COMMENT);
    /* comment */
    checkOnRange(64, 20, 13, TypeOfText.COMMENT);
    /* comment */
  }

  @Test
  @SuppressWarnings("squid:S2699") // ... checkOnRange contains the assertion
  void number() {

    checkOnRange(27, 10, 1, TypeOfText.CONSTANT); //  0
    checkOnRange(28, 10, 1, TypeOfText.CONSTANT); // -1 (without minus)
    checkOnRange(29, 10, 1, TypeOfText.CONSTANT); // +1 (without plus)

    checkOnRange(31, 14, 2, TypeOfText.CONSTANT); // 0u
    checkOnRange(32, 19, 3, TypeOfText.CONSTANT); // 1ul

    checkOnRange(34, 9, 3, TypeOfText.CONSTANT);  // 0x0
    checkOnRange(35, 9, 3, TypeOfText.CONSTANT);  // 0b0
    checkOnRange(36, 9, 16, TypeOfText.CONSTANT); // 0b0100'1100'0110

    checkOnRange(38, 12, 3, TypeOfText.CONSTANT); //  0.0
    checkOnRange(39, 12, 3, TypeOfText.CONSTANT); // -1.0 (without minus)
    checkOnRange(40, 12, 3, TypeOfText.CONSTANT); // +1.0 (without plus)
    checkOnRange(41, 12, 8, TypeOfText.CONSTANT); // 3.14E-10
  }

  @Test
  @SuppressWarnings("squid:S2699") // ... checkOnRange contains the assertion
  void preprocessDirective() {

    checkOnRange(12, 0, 8, TypeOfText.PREPROCESS_DIRECTIVE); // #include

    checkOnRange(14, 0, 6, TypeOfText.PREPROCESS_DIRECTIVE); // #ifdef
    checkOnRange(15, 0, 10, TypeOfText.PREPROCESS_DIRECTIVE); // #   define
    checkOnRange(16, 0, 5, TypeOfText.PREPROCESS_DIRECTIVE); // #else
    checkOnRange(17, 0, 10, TypeOfText.PREPROCESS_DIRECTIVE); // #   define
    checkOnRange(18, 0, 6, TypeOfText.PREPROCESS_DIRECTIVE); // #endif

    checkOnRange(20, 0, 7, TypeOfText.PREPROCESS_DIRECTIVE); // #define
  }

  @Test
  @SuppressWarnings("squid:S2699") // ... checkOnRange contains the assertion
  void identifiersWithSpecialMeaning() {
    // identifier with special meaning => no highlighting
    checkOnRange(112, 11, 6, null); // import
    checkOnRange(119, 10, 6, null); // module
    checkOnRange(120, 12, 6, null); // module
    checkOnRange(122, 13, 6, null); // module
  }

  /**
   * Checks the highlighting of a range of columns. The first column of a line has index 0. The range is the columns of
   * the token.
   */
  private void checkOnRange(int line, int firstColumn, int length, TypeOfText expectedTypeOfText) {
    // check that every column of the token is highlighted (and with the expected type)
    for (var column = firstColumn; column < firstColumn + length; column++) {
      checkInternal(line, column, "", expectedTypeOfText);
    }

    // check that the column before the token is not highlighted
    if (firstColumn != 0) {
      checkInternal(line, firstColumn - 1, " (= before the token)", null);
    }

    // check that the column after the token is not highlighted
    checkInternal(line, firstColumn + length, " (= after the token)", null);
  }

  /**
   * Checks the highlighting of one column. The first column of a line has index 0.
   */
  private void check(int line, int column, TypeOfText expectedTypeOfText) {
    checkInternal(line, column, "", expectedTypeOfText);
  }

  private void checkInternal(int line, int column, String messageComplement, TypeOfText expectedTypeOfText) {
    List<TypeOfText> foundTypeOfTexts = context.highlightingTypeAt(
      "ProjectKey:" + inputFile.file().getName(), line, column);

    int expectedNumberOfTypeOfText = expectedTypeOfText == null ? 0 : 1;
    var message = "number of TypeOfTexts at line " + line + " and column " + column + messageComplement;
    assertThat(foundTypeOfTexts).as(message).hasSize(expectedNumberOfTypeOfText);
    if (expectedNumberOfTypeOfText > 0) {
      message = "found TypeOfTexts at line " + line + " and column " + column + messageComplement;
      assertThat(foundTypeOfTexts.get(0)).as(message).isEqualTo(expectedTypeOfText);
    }
  }

}
