/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java.rules.checkers;


import org.sonar.plugins.java.api.semantic.Symbol;
import org.sonar.plugins.java.api.tree.*;
import org.sonar.check.Rule;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;


import java.util.*;

/**
 * Cookie参数设置中包含敏感字段
 *
 * @author WuHaoYang
 * @date 2024/1/22
 */
@Rule(key = "CookieSensitiveParameterCheck")
public class CookieSensitiveParameterCheck extends IssuableSubscriptionVisitor {



    private static final Set<String> SENSITIVE_COOKIE_PARAMS = new HashSet<>(Arrays.asList("password", "token", "secret"));

    @Override
    public List<Tree.Kind> nodesToVisit() {
        return Arrays.asList(Tree.Kind.METHOD_INVOCATION, Tree.Kind.VARIABLE);
    }

    @Override
    public void visitNode(Tree tree) {
        if (tree.is(Tree.Kind.METHOD_INVOCATION)) {
            visitMethodInvocation((MethodInvocationTree) tree);
        } else if (tree.is(Tree.Kind.VARIABLE)) {
            visitVariable((VariableTree) tree);
        }
    }

    private void visitMethodInvocation(MethodInvocationTree methodInvocation) {
        if (isCookieConstructor(methodInvocation)) {
            checkCookieParameters(methodInvocation);
        }
    }

    private boolean isCookieConstructor(MethodInvocationTree methodInvocation) {
        if (methodInvocation.methodSelect().is(Tree.Kind.MEMBER_SELECT)) {
            MemberSelectExpressionTree memberSelect = (MemberSelectExpressionTree) methodInvocation.methodSelect();
            return memberSelect.expression().symbolType().is("javax.servlet.http.Cookie");
        }
        return false;
    }

    private void visitVariable(VariableTree variableTree) {
        ExpressionTree initializer = variableTree.initializer();
        if (initializer != null && initializer.is(Tree.Kind.NEW_CLASS)) {
            checkCookieParameters(variableTree, ((NewClassTree) initializer).arguments());
        }
    }

    private void checkCookieParameters(MethodInvocationTree methodInvocation) {
        List<? extends ExpressionTree> arguments = methodInvocation.arguments();
        if (arguments != null && arguments.size() == 2) {
            checkSensitiveParameter((ExpressionTree) arguments.get(0), "name", (Symbol) methodInvocation);
            checkSensitiveParameter((ExpressionTree) arguments.get(1), "value", (Symbol) methodInvocation);
        }
    }

    private void checkCookieParameters(VariableTree variableTree, List<? extends ExpressionTree> arguments) {
        Symbol variableSymbol = variableTree.symbol();
        if (arguments != null && arguments.size() == 2) {
            checkSensitiveParameter(arguments.get(0), "name", variableSymbol);
            checkSensitiveParameter(arguments.get(1), "value", variableSymbol);
        }
    }

    private void checkSensitiveParameter(ExpressionTree expression, String paramName, Symbol variableSymbol) {
        if (expression.is(Tree.Kind.IDENTIFIER) && paramName.equals("name")) {
            String variableName = ((IdentifierTree) expression).name();
            if (variableName.equals(variableSymbol.name()) || containsSensitiveParam(variableName)) {
                System.out.println("Cookie参数设置中的name属性与局部变量一致: " + variableName);
                reportIssue(expression, "Cookie参数设置中包含敏感字段");
            }
        } else if (expression.is(Tree.Kind.IDENTIFIER) && paramName.equals("value")) {
            String variableName = ((IdentifierTree) expression).name();
            if (variableName.equals(variableSymbol.name()) || containsSensitiveParam(variableName)) {
                System.out.println("Cookie参数设置中的value属性与局部变量一致: " + variableName);
                reportIssue(expression, "Cookie参数设置中包含敏感字段");
            }
        } else if (expression.is(Tree.Kind.STRING_LITERAL)) {
            String literalValue = LiteralTree.class.cast(expression).value();
            if (containsSensitiveParam(literalValue)) {
                System.out.println("Cookie参数设置中的name属性包含敏感字符串: " + literalValue);
                reportIssue(expression, "Cookie参数设置中包含敏感字段");
            }
        }
    }

    private boolean containsSensitiveParam(String value) {
        String lowerCaseValue = value.toLowerCase();
        return SENSITIVE_COOKIE_PARAMS.stream().anyMatch(lowerCaseValue::contains);
    }
}
