/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * HTTP输入数据验证
 *
 * @author RenFengJiang
 * @date 2024/1/12
 */
@Rule(key = "HttpInputDataChecker")
public class HttpInputDataChecker extends IssuableSubscriptionVisitor {
    @Override
    public List<Tree.Kind> nodesToVisit() {
        /**
         * Tree.Kind.METHOD：方法节点
         * Tree.Kind.BLOCK：方法的代码块节点
         * Tree.Kind.METHOD_INVOCATION： 方法的调用节点
         */
        return Collections.singletonList(Tree.Kind.BLOCK);
    }
    @Override
    public void visitNode(Tree tree) {
        BlockTree blockTree = (BlockTree) tree;
        MethodBodyVisitor methodBodyVisitor = new MethodBodyVisitor(this);
        blockTree.accept(methodBodyVisitor);

    }

    static class MethodBodyVisitor extends BaseTreeVisitor{
        private final HttpInputDataChecker checker;
        List<String> list = new ArrayList<>();
        MethodBodyVisitor(HttpInputDataChecker checker){
            this.checker = checker;
        }

        @Override
        public void visitIfStatement(IfStatementTree tree) {
            ExpressionTree condition = tree.condition();
            if (condition.is(Tree.Kind.EQUAL_TO, Tree.Kind.NOT_EQUAL_TO)) {
                BinaryExpressionTree binaryExpressionTree = (BinaryExpressionTree) condition;

                for (Tree operand : Arrays.asList(binaryExpressionTree.leftOperand(), binaryExpressionTree.rightOperand())) {
                    if (operand.is(Tree.Kind.IDENTIFIER)) {
                        IdentifierTree identifierTree = (IdentifierTree) operand;
                        list.add(identifierTree.name());
                    }
                }
            }
        }

        @Override
        public void visitMethodInvocation(MethodInvocationTree tree) {
            ExpressionTree expressionTree = tree.methodSelect();
            if (expressionTree.is(Tree.Kind.MEMBER_SELECT)) {
                MemberSelectExpressionTree memberSelectExpressionTree = (MemberSelectExpressionTree) expressionTree;
                if ("setHeader".equals(memberSelectExpressionTree.identifier().name())
                        || "addHeader".equals(memberSelectExpressionTree.identifier().name())) {
                    Arguments arguments = tree.arguments();
                    for (ExpressionTree argument : arguments) {
                        if(argument.is(Tree.Kind.STRING_LITERAL)){
                            checker.context.reportIssue(checker, argument, "HTTP输入数据验证");
                            break;
                        } else if(argument.is(Tree.Kind.IDENTIFIER)){
                            IdentifierTree identifierTree = (IdentifierTree) argument;
                            if(!list.contains(identifierTree.name())){
                                checker.context.reportIssue(checker, identifierTree, "HTTP输入数据验证");
                                break;
                            }
                        }
                    }
                }
            }
        }
    }
}
