/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java.rules.checkers;


/**
 * 要求用户使用具有足够复杂度的口令。
 *
 * @author WuHaoyang
 * @date 2024/1/12
 */
import org.sonar.check.Rule;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;

import java.util.*;
import java.util.stream.Collectors;

@Rule(key = "PasswordRegexCheck")
public class PasswordRegexCheck extends IssuableSubscriptionVisitor {

    @Override
    public List<Tree.Kind> nodesToVisit() {
        // 在这里，我们只考虑方法
        return Collections.singletonList(Tree.Kind.METHOD);
    }

    @Override
    public void visitNode(Tree tree) {
        MethodTree method = (MethodTree) tree;
        // 检查是否存在名为 “password”的参数
        List<VariableTree> passwordParameters = method.parameters().stream()
                .filter(param -> "password".equals(param.simpleName().name()))
                .collect(Collectors.toList());

        if (!passwordParameters.isEmpty()) {
            // 确认参数 "password" 是否在正确的正则表达式之下被检查
            boolean passwordIsCheckedWithRegex = passwordParameters.stream()
                    .anyMatch(param -> isPasswordValidatedWithRegex(method, param));

            if (!passwordIsCheckedWithRegex) {
                reportIssue(method.simpleName(), "未对口令进行复杂度验证");
            }
        }
    }


    private boolean isPasswordValidatedWithRegex(Tree tree, VariableTree passwordParameter) {
        PasswordValidationVisitor visitor = new PasswordValidationVisitor(passwordParameter);
        tree.accept(visitor);
        return visitor.isPasswordValidated();
    }

    private static class PasswordValidationVisitor extends BaseTreeVisitor {

        private final VariableTree passwordParameter;
        private boolean passwordValidated = false;

        PasswordValidationVisitor(VariableTree passwordParameter) {
            this.passwordParameter = passwordParameter;
        }

        @Override
        public void visitMethodInvocation(MethodInvocationTree tree) {
            // 改变行为，查找是否存在匹配指定方法调用模式的表达式
            if (isCorrectPasswordValidationMethod(tree)) {
                passwordValidated = true;
            }
            super.visitMethodInvocation(tree);
        }

        public boolean isPasswordValidated() {
            return passwordValidated;
        }

        private boolean isCorrectPasswordValidationMethod(MethodInvocationTree mit) {
            if ("matches".equals(mit.methodSymbol().name())
                    && mit.methodSelect().is(Tree.Kind.MEMBER_SELECT)
                    && ((MemberSelectExpressionTree) mit.methodSelect()).identifier().name().equals("matcher")
                    && ((MemberSelectExpressionTree) mit.methodSelect()).expression().is(Tree.Kind.METHOD_INVOCATION)) {

                MethodInvocationTree innerMit = (MethodInvocationTree) ((MemberSelectExpressionTree) mit.methodSelect()).expression();
                if ("compile".equals(innerMit.methodSymbol().name())
                        && innerMit.methodSelect().is(Tree.Kind.MEMBER_SELECT)
                        && "Pattern".equals( ((MemberSelectExpressionTree)innerMit.methodSelect()).expression().toString())) {
                    // 需要确定参数中包含名为 'password' 的参数
                    List<VariableTree> arguments = mit.methodSymbol().declaration().parameters().stream().filter(param -> param.symbol().name().equals("password")).collect(Collectors.toList());
                    return (!arguments.isEmpty());
                }
            }
            return false;
        }
    }

}
