/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.semantic.Symbol;
import org.sonar.plugins.java.api.semantic.Type;
import org.sonar.plugins.java.api.tree.*;

import javax.annotation.Nonnull;
import java.util.List;

/**
 * 将页面隐藏域字段、Cookie、URL等关键参数缓存到服务器端的会话中，程序使用该数据须通过会话获取
 * <p>在Java web应用开发中，隐藏域字段、Cookie、URL等关键参数应通过会话获取和传递。</p>
 *
 * @author GuoXin
 * @date 2024/1/24
 */
@Rule(key = "SessionCacheParamsChecker")
public class SessionCacheParamsChecker extends IssuableSubscriptionVisitor {

    //违规集合
    private static final List<String> HIDED_PARAMS = List.of(
            "userid",
            "token",
            "url",
            "userpassword"
    );
    private static final String requestType = "javax.servlet.http.HttpServletRequest";

    @Override
    public List<Tree.Kind> nodesToVisit() {
        return List.of(Tree.Kind.METHOD_INVOCATION);
    }

    @Override
    public void visitNode(@Nonnull Tree tree) {
        MethodInvocationTree methodInvocationTree = (MethodInvocationTree) tree;
        Symbol.MethodSymbol methodSymbol = (Symbol.MethodSymbol) methodInvocationTree.methodSymbol();
        //获取参数
        ExpressionTree expressionTree = methodInvocationTree.methodSelect();
        if(expressionTree.is(Tree.Kind.MEMBER_SELECT)){
            MemberSelectExpressionTree selectExpressionTree = (MemberSelectExpressionTree) expressionTree;
            //获取参数类型
            Type type = selectExpressionTree.expression().symbolType();
            if(type != null){
                //判断参数类型和调用方法符不符合要求
                if(requestType.equals(type.fullyQualifiedName())
                        && ("getParameter".equals(methodSymbol.name()) || "getCookies".equals(methodSymbol.name()))) {
                    //如果是getCookies方法直接抛错
                    if("getCookies".equals(methodSymbol.name())){
                        reportIssue(methodInvocationTree, "页面隐藏域字段、Cookie、URL等关键参数不能直接获取，应缓存到服务器端的会话中并通过会话获取");
                    }else {
                        //获取参数
                        Arguments arguments = methodInvocationTree.arguments();
                        for (ExpressionTree argument : arguments) {
                            if(argument.is(Tree.Kind.STRING_LITERAL)){
                                String name = ((LiteralTree) argument).value()
                                        .replace("\"", "").toLowerCase();
                                //判断是否是违规项
                                if(HIDED_PARAMS.contains(name)){
                                    System.out.println(methodInvocationTree);
                                    reportIssue(methodInvocationTree, "页面隐藏域字段、Cookie、URL等关键参数不能直接获取，应缓存到服务器端的会话中并通过会话获取");
                                    break;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
