/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */

package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;

import java.util.List;

/**
 * 对用户进行身份鉴别并建立一个新的会话时让原来的会话失效
 *
 * @author RenFengJiang
 * @date 2024/1/24
 */
@Rule(key = "UserStatusVerifyChecker")
public class UserStatusVerifyChecker extends IssuableSubscriptionVisitor {
    @Override
    public List<Tree.Kind> nodesToVisit() {
        return List.of(Tree.Kind.METHOD);
    }

    @Override
    public void visitNode(Tree tree) {
        //将Tree强转成MethodTree
        MethodTree methodTree = (MethodTree) tree;
        //判断是否是doFilter或doFilter方法
        boolean boo = verifyMethod(methodTree);
        if(boo){
            //内部实现类
            VisitMethod visitMethod = new VisitMethod(this);
            //调用内部实现类
            methodTree.block().accept(visitMethod);
            //判断是否满足4个条件
            if(visitMethod.booOne && visitMethod.booTwo && visitMethod.booThree && visitMethod.booFour){

            }else{
                System.out.println(methodTree.simpleName());
                context.reportIssue(this, methodTree.simpleName(), "对用户进行身份鉴别并建立一个新的会话时让原来的会话失效");
            }

        }
    }

    public static boolean verifyMethod(MethodTree methodTree) {
        //preHandle
        if ("doFilter".equals(methodTree.simpleName().name()) || "preHandle".equals(methodTree.simpleName().name())) {
            //获取参数
            List<VariableTree> parameters = methodTree.parameters();
            for (VariableTree variable : parameters) {
                if (variable.type().symbolType().name().endsWith("ServletRequest")) {
                    return true;
                }
            }
        }
        return false;
    }

    static class VisitMethod extends BaseTreeVisitor {
        private final UserStatusVerifyChecker checker;

        private boolean booOne = false;
        private boolean booTwo = false;
        private boolean booThree = false;
        private boolean booFour = false;


        public VisitMethod(UserStatusVerifyChecker checker) {
            this.checker = checker;
        }

        //方法调用节点
        @Override
        public void visitMethodInvocation(MethodInvocationTree tree) {
            ExpressionTree expressionTree = tree.methodSelect();
            if(expressionTree.is(Tree.Kind.MEMBER_SELECT)){
                //判断是否调用指定的方法
                MemberSelectExpressionTree selectExpressionTree = (MemberSelectExpressionTree) expressionTree;
                if("getParameter".equals(selectExpressionTree.identifier().name()) || "getSession".equals(selectExpressionTree.identifier().name())){
                    //获取到调用方法的参数
                    Arguments arguments = tree.arguments();
                    //调用判断参数的方法
                    verifyParam(arguments);
                }else if ("invalidate".equals(selectExpressionTree.identifier().name())){
                    booFour = true;
                }
            }
        }

        public void verifyParam(Arguments arguments){
            for (ExpressionTree argument : arguments) {
                if(argument.is(Tree.Kind.STRING_LITERAL)){
                    LiteralTree literalTree = (LiteralTree) argument;
                    String strName = literalTree.value().replace("\"", "").toLowerCase();
                    if("username".equals(strName)){
                        booOne = true;
                    }else if("password".equals(strName)){
                        booTwo = true;
                    }else if("false".equals(strName)){
                        booThree = true;
                    }
                }
            }
        }
    }
}
